""" basic commands. """

import logging
import json
import os
import time

from obj.base import Dotted, Log, cfg, get_type, get_types, starttime
from obj.loader import Loader, aliases
from obj.store import Store
from obj.tasks import kill, running
from obj.utils import days, elapsed, get_exception, short, stripl

def ed(event):
    if not cfg.workdir:
        cfg.workdir = hd(".obj")
        cdir(cfg.workdir)
    if not event.args:
        event.reply("|".join(os.listdir(cfg.workdir)))
        return
    types = get_types(event.type)
    if not types:
        event.reply("ETYPE %s" % event.type)
        return
    if len(types) > 1:
        event.reply("|".join(types))
        return
    store = Store()
    o = store.last(types[0])
    if not o:
        try:
            o = get_cls(event.type)()
        except ModuleNotFoundError:
            event.reply("not found %s" % event.type)
            return
    if not event.setter:
        event.reply(o)
        return
    for key, value in event.setter.items():
        val = None
        v = None
        try:
            if "," in value:
                value = value.split(",")
            if type(value) == list:
                v = "%s" % str(value)
                v = v.replace("'",'"')
            else:
                v = '"%s"' % value
            val = json.loads(v)
        except json.decoder.JSONDecodeError:
            event.reply("%s is not JSON" % value)
            continue
        try:
            val = int(val)
        except (TypeError, ValueError):
            try:
                val = float(val)
            except (TypeError, ValueError):
                pass
        t = type(val)
        if val in ["True", "true"]:
            o.set_attr(key, True)
        elif val in ["False", "false"]:
            o.set_attr(key, False)
        elif t == list:
            o.set_attr(key, val)
        elif t == str:
            o.set_attr(key, val)
        else:
            o[key] = val
    fn = o.save()
    event.reply("ok %s" % fn)

def find(event):
    if not event.args:
        event.reply("|".join(short(get_types())))
        return
    store = Store()
    types = get_types(event.type)
    if len(types) > 1:
        event.reply("|".join(short(types)))
        return
    if not types:
        event.reply("no matching types.")
        return
    t = types[0]
    if "k" in event.options:
        o = store.last(t)
        if o:
            event.reply("|".join(stripl(o)))
            return
    if not event.selector:
        func = store.all
    else:
        func = store.find
    stime = time.time()
    res = list(func(t, event.selector, event.index, event.delta))
    nr = -1
    for o in res:
        if not o:
            event.reply("ETYPE %s" % event.args[0])
            continue
        txt = ""
        if "d" in event.options:
            event.reply(str(o))
            continue
        full = False
        if "f" in event.options:
            full = True
        nr += 1
        if event.dkeys:
            txt = "%s %s" % (event.index or nr, o.format(event.dkeys, full))
        else:
            txt = "%s %s" % (event.index or nr, o.format(full=full))
        if "t" in event.options:
            txt += " " + days(o)
        event.reply(txt)
    logging.warn("ok %s %s" % (len(res), elapsed(time.time()-stime)))

def log(event):
    log = Log()
    log.txt = event.rest
    p = log.save()
    event.reply("ok %s" % p)

def ps(event):
    psformat = "%-8s %-60s"
    result = []
    try:
        getnr = int(event.args[0])
    except (ValueError, IndexError):
        getnr = None
    for thr in sorted(running(), key=lambda x: x.getName()):
        obj = Dotted()
        obj.update(vars(thr))
        if getattr(obj, "sleep", None):
            up = obj.sleep - int(time.time() - obj.state.latest)
        else:
            up = int(time.time() - starttime)
        result.append((up, thr.getName(), obj))
    nr = -1
    for up, thrname, o in sorted(result, key=lambda x: x[0]):
        nr += 1
        if getnr is not None and getnr == nr:
            event.reply(o)
            return
        res = "%s %s" % (nr, psformat % (elapsed(up), thrname[:30]))
        event.reply(res)

def rm(event):
    if not event.args:
        event.reply("|".join(sorted(get_types())))
        return
    st = time.time()
    nr = -1
    store = Store()
    for o in store.find(event.type, event.selector, event.index, event.delta):
        nr += 1
        o._deleted = True
        o.save()
    event.reply("ok %s %s" % (nr+1, elapsed(time.time()-st)))

def show(event):
    try:
        cmd, *args = event.args
    except ValueError:
        event.reply("cfg|cmds|keys|mods|uptime|version")
        return
    if cmd == "cfg":
        if len(args):
            type = "obj.%s.Cfg" % args[0]
            event.reply(store.last(type))
        else:
            event.reply(cfg)
    elif cmd == "cmds":
        loader = Loader()
        loader.walk("obj")
        if args:
            event.reply(",".join(sorted([x for x in loader.names.keys() if args[0] in str(x)])))
        else:
            event.reply(",".join(sorted(loader.names.keys())))
    elif cmd == "keys":
        if not args:
            res = {x.split(".")[-1].lower() for x in os.listdir(cfg.workdir)}
            event.reply("|".join(sorted(res)))
            return
        for otype in get_types(args[0]):
            l = store.last(otype)
            if l:
                res = [x for x in l.keys() if not x.startswith("_")]
                event.reply("|".join(res))
    elif cmd == "mods":
        l = Loader()
        l.walk("obj")
        res = {x.split(".")[-1].lower() for x in l.table.keys()}
        event.reply("|".join(sorted(res)))
    elif cmd == "help":
        res = ""
        for key, val in aliases.items():
            res += "%s (%s) " % (key, val.split()[-1])
        if res:
            event.reply(res)
    elif cmd == "types":
        event.reply("|".join(sorted({x.split(".")[-1].lower() for x in os.listdir(cfg.workdir)})))
    elif cmd == "version":
        event.reply("%s %s" % (cfg.name.upper(), cfg.version))
    elif cmd == "uptime":
        event.reply(elapsed(time.time() - starttime))

def stop(event):
    if not event.args:
        event.reply("stop <threadname>")
        return
    name = event.args[0]
    kill(name)
    event.reply("ok %s" % name)

def undel(event):
    if not event.args:
        event.reply("|".join(sorted(get_types())))
        return
    st = time.time()
    nr = -1
    store = Store()
    for o in store.all(event.type, event.selector, event.index, event.delta, showdel=True):
        nr += 1
        o._deleted = False
        o.save()
    event.reply("ok %s %s" % (nr+1, elapsed(time.time()-st)))
