""" list of tokens parsed into a command. """

import threading

from obj.base import Dotted, cfg, get_types
from obj.utils import get_time, parse_date, to_day

class Token(Dotted):

    def __init__(self, word):
        super().__init__()
        self.arg = ""
        self.dkey = ""
        self.down = None
        self.equal = ""
        self.ignore = ""
        self.index = None
        self.options = []
        self.selector = ""
        self.setter = ""
        self.up = None
        self.value = ""
        self.word = word
        self.parse()

    def parse(self):
        if "http" in self.word:
            self.value = self.word
            self.arg = self.word
            return
        try:
            v = int(self.word)
            if v >= 0:
                self.index = v 
                return
        except:
            pass
        if self.word.startswith("-"):
            try:
                self.down = int(self.word)
                return
            except ValueError:
                pass
        if self.word.startswith("+"):
            try:
                self.up = int(self.word[1:])
                return
            except ValueError:
                pass
        if self.word.endswith("-"):
            self.ignore = self.word[:-1]
            self.word = self.ignore
        if "==" in self.word:
            self.selector, self.value = self.word.split("==")
            self.dkey = self.selector
        elif "=" in self.word:
            self.setter, self.value = self.word.split("=")
            self.dkey = self.setter
        else:
            self.arg = self.word
            self.value = self.word

class Command(Dotted):

    def __init__(self, txt):
        super().__init__()
        self._func = None
        self._ready = threading.Event()
        self._result = []
        self._thrs = []
        self.args = []
        self.channel = ""
        self.cmd = ""
        self.delta = 0
        self.dkeys = []
        self.ignore = ""
        self.index = None
        self.options = []
        self.rest = ""
        self.selector = Dotted()
        self.setter = Dotted()
        self.tokens = []
        self.txt = txt
        self.parse()

    def ok(self, txt=""):
        self.reply("ok %s" % txt or self.cmd.cmd)

    def parse(self):
        nr = -1
        self.args = []
        self.dkeys = []
        self.options = [x for x in cfg.options.split(",") if x]
        self.tokens = []
        if not self.txt:
            return
        words = self.txt.split()
        for word in words:
            nr += 1
            if nr == 0:
                if word.startswith("!"):
                    word = word[1:]
                self.cmd = word
                continue
            if nr == 1:
                self.args.append(word)
                self.type = word
                continue
            token = Token(word)
            self.tokens.append(token)
        nr = -1
        prev = ""
        for token in self.tokens:
             nr += 1
             if prev == "-o":
                 self.options.extend([x for x in token.value.split(",") if x])
             if prev == "-i":
                 try:
                     self.index = int(token.value)
                 except ValueError:
                     pass
             if token.arg:
                 self.args.append(token.arg)
             if token.ignore:
                 self.ignore = token.ignore
             elif token.dkey:
                 self.dkeys.append(token.dkey)
             elif token.arg:
                 self.dkeys.append(token.arg)
             if token.selector:
                 self.selector[token.selector] = token.value
             if token.setter:
                 self.setter[token.setter] = token.value
             if token.index is not None:
                 self.index = token.index
             if token.up:
                 self.delta = parse_date(token.up)
             if token.down:
                 self.delta = parse_data(token.down)
             prev = token.value
        self.rest = " ".join(self.args)
        self.time = to_day(self.rest)

    def ready(self):
        return self._ready.set()

    def reply(self, txt):
        self._result.append(txt)

    def show(self, obj):
        for txt in self._result:
            obj.say(self.orig, self.channel, txt)

    def wait(self):
        self._ready.wait()
        result = []
        thrs = []
        for thr in self._thrs:
            try:
                ret = thr.join()
            except RuntimeError:
                continue
            thrs.append(thr)
            self._result.append(ret)
        for thr in thrs:
            self._thrs.remove(thr)
        return self
