""" event handler. """

import logging
import queue
import sys
import time
import threading

from obj.base import Dotted, get_type
from obj.loader import Loader
from obj.tasks import launch
from obj.utils import get_name

class ENOTIMPLEMENTED(Exception):
    pass

class Handler(Loader):

    def __init__(self):
        super().__init__()
        self._curevent = None
        self._queue = queue.Queue()
        self._ready = threading.Event()
        self._stopped = False
        self._threaded = True
        self._thrs = []
        self.state = Dotted()
        self.verbose = False

    def collect(self):
        res = []
        for thr in self._thrs:
            res.append(thr.join())
        return res

    def dispatch(self, event):
        logging.warn("dispatch %s" % event.txt)
        event._func = self.get_cmd(event)
        if event._func:
            event._func(event)
            event.show(self)
        event.ready()

    def get_cmd(self, event):
        return self.get_handler(event.cmd)

    def get_event(self):
        return self._queue.get()

    def loop(self):
        while not self._stopped:
            try:
                event = self.get_event()
            except EOFError:
                break
            if not event:
                continue
            if self._threaded:
                thr = launch(self.dispatch, event)
                event._thrs.append(thr)
            else:
                self.dispatch(event)
        self.ready()

    def put(self, event):
        self._queue.put_nowait(event)

    def ready(self):
        self._ready.set()

    def say(self, orig, channel, txt):
        if self.verbose:
            print(txt)

    def start(self):
        launch(self.loop)

    def stop(self):
        self._stopped = True
        self._queue.put(None)

    def wait(self):
        return self._ready.wait()

    def work(self):
        while not self._stopped:
            event = self._queue.get()
            if not event:
                break
            self.dispatch(event)
        self.ready()
