""" module loader. """

import logging
import importlib
import os
import pkgutil
import time
import types

from obj.base import Dotted, cfg, starttime
from obj.tasks import launch, running
from obj.utils import elapsed, get_exception, get_name

aliases = {
           "c": "show cmds",
           "cfg": "show cfg",
           "f": "find",
           "h": "show help",
           "l": "load",
           "m": "show mods",
           "s": "show",
           "t": "show types",
           "u": "unload",
           "v": "show version",
}

names = {
         "ed": "obj.cmds",
         "find": "obj.cmds",
         "log": "obj.cmds",
         "ps": "obj.cmds",
         "rm": "obj.cmds",
         "show": "obj.cmds",
         "stop": "obj.cmds",
         "undel": "obj.cmds",
}

class Loader(Dotted):

    table = Dotted()
    names = names

    def __init__(self):
        super().__init__()
        self.handlers = Dotted()
       
    def cached(self, name):
        if name not in Loader.table:
            Loader.table[name] = self.direct(name) 
        return Loader.table[name]

    def direct(self, name, package=None):
        logging.warn("load %s" % name)
        return importlib.import_module(name, package)

    def get_aliased(self, txt):
        spl = txt.split()
        if spl and spl[0] in aliases:
            cmd = spl[0]
            v = aliases.get(cmd, None)
            if v:
                spl[0] = cmd.replace(cmd, v)
        txt = " ".join(spl)
        return txt

    def get_handler(self, cmd):
        val = self.handlers.get(cmd, None)
        if not val:
            modname = Loader.names.get(cmd, None)
            if modname:
                self.load_mod(modname)
            val = self.handlers.get(cmd, None)
        return val

    def init(self, m):
        func = getattr(m, "init", None)
        if func:
            thr = launch(func)
            return thr

    def load_mod(self, name, mod=None):
        mod = mod or self.cached(name)
        self.table[name] = mod
        for key, func, modname in self.scan(mod):
            self.handlers[key] = func
            Loader.names[key] = modname
        return mod

    def register(self, cmd, handler):
        self.handlers[cmd] = handler

    def scan(self, mod):
        for key in dir(mod):
            if key.startswith("_"):
                continue
            func = getattr(mod, key, None)
            if func and isinstance(func, types.FunctionType):
                if "event" in func.__code__.co_varnames:
                    yield (key, func, mod.__name__)

    def unload(self, modname):
        mod = self.table[modname]
        for key, func, name in self.scan(mod):
            if name == modname:
                try:
                    del self.handlers[key]
                    del Loader.names[key]
                except KeyError:
                    pass

    def walk(self, modname, init=False):
        mods = []
        mod = self.load_mod(modname)
        if "__path__" in dir(mod):
            for nn in pkgutil.walk_packages(mod.__path__, modname+'.'):
                try:
                    mods.append(self.load_mod(nn[1]))
                except Exception as ex:
                    logging.error(get_exception())
        mods.append(mod)
        thrs = []
        if init:
            for m in mods:
                if "init" in dir(m):
                    thrs.append(launch(m.init))
        return thrs
