""" timestamped JSON file backend. """

import importlib
import json
import logging
import os
import time

from obj.base import Dotted, cfg, get_type, get_types, starttime
from obj.utils import construct, days, elapsed, fn_time, get_cls, get_exception

class Store(Dotted):

    def all(self, otype, selector=None, index=None, delta=0, showdel=False):
        nr = -1
        for fn in sorted(filenames(otype, delta)):
            nr += 1
            try:
                obj = construct(fn)
            except ModuleNotFoundError:
                continue
            if not showdel and ("_deleted" in obj and obj._deleted):
                continue
            if index is not None and nr != index:
                continue
            yield obj

    def find(self, otype, selector=None, index=None, delta=0, showdel=False):
        nr = -1
        for fn in sorted(filenames(otype, delta)):
            print(fn)
            nr += 1
            try:
                o = construct(fn)
            except ModuleNotFoundError:
                continue
            except Exception:
                logging.error(get_exception())
                continue
            if not o:
                continue
            if not showdel and ("_deleted" in dir(o) and o._deleted):
                continue
            if o.search(selector):
                if index is not None and nr != index:
                    continue
                yield o

    def last(self, otype):
        fns = sorted(filenames(otype), key=lambda x: fn_time(x))
        if fns:
            try:
                return construct(fns[-1])
            except ModuleNotFoundError:
                return None

    def last_fn(self, otype):
        fns = sorted(filenames(otype), key=lambda x: fn_time(x))
        if fns:
            return fns[-1]

def filenames(ftype, delta=0):
    p = os.path.join(cfg.workdir, ftype)
    now = time.time()
    past = now + delta
    for rootdir, dirs, files in os.walk(p, topdown=True):
        if p not in rootdir:
            continue
        for fn in files:
            fnn = os.path.join(rootdir, fn).split(cfg.workdir)[-1][1:]
            if delta:
                if fn_time(fnn) < past:
                    continue
            yield fnn

def fn_time(daystr):
    daystr = daystr.replace("_", ":")
    datestr = " ".join(daystr.split(os.sep)[-2:])
    datestr = datestr.split(".")[0]
    try:
        t = time.mktime(time.strptime(datestr, "%Y-%m-%d %H:%M:%S"))
    except ValueError:
        t = 0
    return t

def get_fntype(fn):
    return fn.split(os.sep)[0]
